package eu.mhsl.craftattack.spawn.core.api.client.repositories;

import eu.mhsl.craftattack.spawn.core.api.client.HttpRepository;
import eu.mhsl.craftattack.spawn.core.api.client.ReqResp;
import eu.mhsl.craftattack.spawn.core.util.api.WebsiteApiUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.UUID;

public class ReportRepository extends HttpRepository {
    public ReportRepository() {
        super(WebsiteApiUtil.getBaseUri(), WebsiteApiUtil::withAuthorizationSecret);
    }

    public record ReportCreationInfo(@NotNull UUID reporter, @Nullable UUID reported, String reason) {
    }

    public record ReportUrl(@NotNull String url) {
    }

    public record PlayerReports(
        List<Report> from_self,
        Object to_self
    ) {
        public record Report(
            @Nullable Reporter reported,
            @NotNull String subject,
            boolean draft,
            @NotNull String status,
            @NotNull String url
        ) {
            public record Reporter(
                @NotNull String username,
                @NotNull String uuid
            ) {
            }
        }
    }

    public ReqResp<PlayerReports> queryReports(UUID player) {
        return this.get(
            "report",
            (parameters) -> parameters.addParameter("uuid", player.toString()),
            PlayerReports.class
        );
    }

    public ReqResp<ReportUrl> createReport(ReportCreationInfo data) {
        return this.post(
            "report",
            data,
            ReportUrl.class
        );
    }
}
