package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.util.text.DisconnectInfo;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.displayName.DisplayName;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.joinProtection.JoinProtection;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

public class Teams extends Appliance implements DisplayName.Prefixed, DisplayName.Colored {
    private Set<VaroTeam> teams = new HashSet<>() {{
        this.add(new VaroTeam(
            new HashSet<>() {{
               this.add(new VaroTeam.Member(UUID.fromString("3a0a699c-7f0c-319b-839c-ccc8cac8ae63"), false));
               this.add(new VaroTeam.Member(UUID.fromString("959ed433-14ea-38fe-918b-75b7d09466af"), true));
            }},
            "Die Admins",
            "#0E0E0E"
        ));
    }};

    public void refreshTeamList() {

    }

    public boolean canLogin(UUID playerId) {
        return this.teams.stream()
            .anyMatch(varoTeam -> varoTeam.hasMember(playerId) && !Objects.requireNonNull(varoTeam.getMemberById(playerId)).isDead);
    }

    VaroTeam getTeamFromPlayer(UUID playerId) {
        return this.teams.stream()
            .filter(varoTeam -> varoTeam.hasMember(playerId))
            .findFirst()
            .orElseThrow();
    }

    public void enforceTeamJoin(Player joinedPlayer) {
        this.getTeamFromPlayer(joinedPlayer.getUniqueId()).joinCountdown = new VaroTeam.JoinCountdown();

        DisconnectInfo disconnectInfo = new DisconnectInfo(
            "Teampartner nicht beigetreten",
            "Deine Verbindung wurde getrennt, da dein Teampartner keine Verbindung zum Server hergestellt hat!",
            "Bitte sorge dafür, dass alle anderen Teammitglieder eine einwandfreie Internetverbindung haben und melde dich im Zweifel bei einem Admin!",
            joinedPlayer.getUniqueId()
        );

        VaroTeam team = this.getTeamFromPlayer(joinedPlayer.getUniqueId());
        Bukkit.getScheduler().scheduleSyncDelayedTask(
            Main.instance(),
            () -> {
                team.members.stream()
                    .filter(member -> !member.isDead)
                    .filter(member -> {
                        Player p = Bukkit.getPlayer(member.player.getUniqueId());
                        return p == null || !p.isOnline();
                    })
                    .findAny()
                    .ifPresent(member -> team.kickTeam(disconnectInfo));
            },
            Ticks.TICKS_PER_SECOND * (JoinProtection.resistanceDuration / 2)
        );
    }

    public void enforceTeamLeave(Player leftPlayer) {
        DisconnectInfo disconnectInfo = new DisconnectInfo(
            "Dein Teampartner hat den Server vorzeitig verlassen",
            "Dein Team erhält daher einen Strike!",
            "Bitte bleibt stets die vollen 30 Minuten auf dem Server!",
            leftPlayer.getUniqueId()
        );
        VaroTeam team = this.getTeamFromPlayer(leftPlayer.getUniqueId());
        Bukkit.getScheduler().scheduleSyncDelayedTask(
            Main.instance(),
            () -> team.kickTeam(disconnectInfo),
            Ticks.TICKS_PER_SECOND
        );
    }

    public VaroTeam.JoinCountdown getTeamJoinCountdown(Player player) {
        return this.getTeamFromPlayer(player.getUniqueId()).joinCountdown;
    }

    @Override
    public @Nullable TextColor getNameColor(Player player) {
        return NamedTextColor.WHITE ;
    }

    @Override
    public @Nullable Component getNamePrefix(Player player) {
        VaroTeam team = this.getTeamFromPlayer(player.getUniqueId());
        return Component.text(
            String.format("[%s]", team.name),
            TextColor.fromCSSHexString(team.color)
        );
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new ConnectivityChangeListener());
    }
}
